@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2012)
+#   Copyright (C) Dr. Heinz-Josef Claes (2008-2013)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -114,8 +114,10 @@ or corrupted
 =head1 SYNOPSIS
 
 	storeBackupCheckBackup.pl -c backupDir [-p number] [-i]
+	      [-w filePrefix] [--lastOfEachSeries]
+	      [--includeRenamedBackups]
 	      [--logFile
-	       [--suppressTime] [-m maxFilelen]
+	       [--plusLogStdout] [--suppressTime] [-m maxFilelen]
 	       [[-n noOfOldFiles] | [--saveLogs]]
 
 =head1 DESCRIPTION
@@ -152,15 +154,35 @@ It only checks plain files, not special files or symbolic links.
 
     number of parallel jobs, default = chosen automatically
 
+=item B<--lastOfEachSeries>
+
+    only check the last backup of each series found
+
 =item B<--includeRenamedBackups>, B<-i>
 
     include renamed backups into the check renamed backups must
     follow the convention <backupDir>-<something>
 
+=item B<--wrongFileTables>, B<-w>
+
+    write filenames with detected faults in regular files for
+    later bug fixing (not automated)
+    parameter to this option is a file prefix
+    eg. if the file prefix is '/tmp/bugsB-', the following files are
+    generated:
+    /tmp/bugsB-files.missing.txt
+    /tmp/bugsB-md5sums.missing.txt
+    /tmp/bugsB-md5sums.wrong.txt
+
 =item B<--logFile>, B<-l>
 
     logFile, Default: stdout
 
+=item B<--plusLogStdout>
+
+    if you specify a log file with --logFile you can
+    additionally print the output to STDOUT with this flag
+
 =item B<--suppressTime>
 
     suppress output of time in logfile
@@ -187,12 +209,12 @@ It only checks plain files, not special files or symbolic links.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008-2012 by Heinz-Josef Claes (see README)
+Copyright (c) 2008-2013 by Heinz-Josef Claes (see README)
 Published under the GNU General Public License v3 or any later version
 
 =cut
 
-my $Help = join('', grep(!/^\s*$/, `pod2text $0`));
+my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
 $Help = "cannot find pod2text, see documentation for details\n"
     unless $Help;
 
@@ -214,15 +236,24 @@ my $CheckPar =
 					    '-pattern' => '\A[1-9]\d*\Z'),
 				Option->new('-name' => 'print',
 					    '-cl_option' => '--print'),
+				Option->new('-name' => 'lastOfEachSeries',
+					    '-cl_option' => '--lastOfEachSeries'),
 				Option->new('-name' => 'includeRenamedBackups',
 					    '-cl_option' => '-i',
 					    '-cl_alias' =>
 					    '--includeRenamedBackups'),
+				Option->new('-name' => 'wrongFileTables',
+					    '-cl_option' => '-w',
+					    '-cl_alias' => '--wrongFileTables',
+					    '-param' => 'yes'),
 				Option->new('-name' => 'logFile',
 					    '-cl_option' => '-l',
 					    '-cl_alias' => '--logFile',
 					    '-param' => 'yes',
 					    '-only_if' => 'not [interactive]'),
+				Option->new('-name' => 'plusLogStdout',
+					    '-cl_option' => '--plusLogStdout',
+					    '-only_if' => '[logFile]'),
 				Option->new('-name' => 'suppressTime',
 					    '-cl_option' => '--suppressTime'),
 				Option->new('-name' => 'maxFilelen',
@@ -245,7 +276,13 @@ my $CheckPar =
 					    '-cl_option' => '--compressWith',
 					    '-quoteEval' => 'yes',
                                             '-default' => 'bzip2',
-                                            '-only_if' =>'[logFile]')
+                                            '-only_if' =>'[logFile]'),
+# hidden options
+# used by storeBackupMount.pl
+				Option->new('-name' => 'writeToNamedPipe',
+					    '-cl_option' => '--writeToNamedPipe',
+					    '-param' => 'yes',
+					    '-hidden' => 'yes')
 		    ]
     );
 
@@ -259,9 +296,12 @@ my $print = $CheckPar->getOptWithoutPar('print');
 my $backupDir = $CheckPar->getOptWithPar('checkBackup');
 my $parJobs = $CheckPar->getOptWithPar('parJobs');
 my $verbose = $CheckPar->getOptWithoutPar('verbose');
+my $lastOfEachSeries = $CheckPar->getOptWithoutPar('lastOfEachSeries');
 my $includeRenamedBackups =
     $CheckPar->getOptWithoutPar('includeRenamedBackups');
+my $wrongFileTables = $CheckPar->getOptWithPar('wrongFileTables');
 my $logFile = $CheckPar->getOptWithPar('logFile');
+my $plusLogStdout = $CheckPar->getOptWithoutPar('plusLogStdout');
 my $withTime = not $CheckPar->getOptWithoutPar('suppressTime');
 $withTime = $withTime ? 'yes' : 'no';
 my $maxFilelen = $CheckPar->getOptWithPar('maxFilelen');
@@ -269,6 +309,8 @@ my $noOfOldFiles = $CheckPar->getOptWithPar('noOfOldFiles');
 my $saveLogs = $CheckPar->getOptWithPar('saveLogs');
 $saveLogs = $saveLogs ? 'yes' : 'no';
 my $compressWith = $CheckPar->getOptWithPar('compressWith');
+# hidden options
+my $writeToNamedPipe = $CheckPar->getOptWithPar('writeToNamedPipe');
 
 
 unless ($parJobs)
@@ -314,15 +356,29 @@ my ($prLogKind) = ['A:BEGIN',
 		   'E:ERROR',
 		   'S:STATISTIC',
 		   'D:DEBUG'];
-my $printLog = printLog->new('-kind' => $prLogKind,
-			     @par,
-			     '-withTime' => $withTime,
-			     '-maxFilelen' => $maxFilelen,
-			     '-noOfOldFiles' => $noOfOldFiles,
-			     '-saveLogs' => $saveLogs,
-			     '-compressWith' => $compressWith);
+my $prLog1 = printLog->new('-kind' => $prLogKind,
+			   @par,
+			   '-withTime' => $withTime,
+			   '-maxFilelen' => $maxFilelen,
+			   '-noOfOldFiles' => $noOfOldFiles,
+			   '-saveLogs' => $saveLogs,
+			   '-compressWith' => $compressWith);
 
-my $prLog = printLogMultiple->new('-prLogs' => [$printLog]);
+my $prLog = printLogMultiple->new('-prLogs' => [$prLog1]);
+
+if ($plusLogStdout)
+{
+    my $p = printLog->new('-kind' => $prLogKind,
+			  '-filedescriptor', *STDOUT);
+    $prLog->add('-prLogs' => [$p]);
+}
+if ($writeToNamedPipe)
+{
+    my $np = printLog->new('-kind' => $prLogKind,
+			   '-file' => $writeToNamedPipe,
+			   '-maxFilelen' => 0);
+    $prLog->add('-prLogs' => [$np]);
+}
 
 $prLog->fork($req);
 
@@ -344,79 +400,22 @@ $prLog->print('-kind' => 'A',
 $prLog->print('-kind' => 'V',
 	      '-str' => ["storeBackupCheckBackup.pl, $main::STOREBACKUPVERSION"]);
 
-if (-l $backupDir)
-{
-    my $newLink = ::absolutePath($backupDir);
-    $prLog->print('-kind' => 'I',
-		  '-str' =>
-		  ["replacing symlink <$backupDir> with <$newLink>"]);
-    $backupDir = $newLink;
-}
+my (@dirsToCheck) = &::selectBackupDirs($backupDir, $includeRenamedBackups,
+					$checkSumFile, $prLog,
+					$lastOfEachSeries);
 
+(@main::cleanup) = ($prLog, undef);
+$SIG{INT} = \&cleanup;
+$SIG{TERM} = \&cleanup;
 
-my $allLinks = lateLinks->new('-dirs' => [$backupDir],
-			      '-kind' => 'recursiveSearch',
-			      '-verbose' => 0,
-			      '-prLog' => $prLog,
-			      '-includeRenamedBackupDirs' =>
-			      $includeRenamedBackups);
-
-my $allStbuDirs = $allLinks->getAllStoreBackupDirs();
-
-
-# filter the relevant backups
-my (@dirsToCheck) = sort { $a cmp $b } @$allStbuDirs;
-
-
-$prLog->print('-kind' => 'E',
-	      '-str' => ["nothing to search, no backup directories specified"],
-	      '-exit' => 1)
-    unless @dirsToCheck;
-
-{
-    my (@out, @d, $d);
-    (@d) = (@dirsToCheck);
-    (@dirsToCheck) = ();
-    foreach $d (@d)
-    {
-	unless (-r "$d/$checkSumFile" or
-		-r "$d/$checkSumFile.bz2")
-	{
-	    $prLog->print('-kind' => 'E',
-			  '-str' => ["no readable <$checkSumFile> in " .
-				     "<$d> ... skipping"]);
-	    next;
-	}
-	if (-e "$d/$checkSumFile.notFinished")
-	{
-	    $prLog->print('-kind' => 'E',
-			  '-str' => ["backup <$d> not finished" .
-				     " ... skipping"]);
-	    next;
-	}
-	if (-e "$d/.storeBackupLinks/linkFile.bz2")
-	{
-	    $prLog->print('-kind' => 'E',
-			  '-str' =>
-			  ["backup <$d> needs run of " .
-			   "~/svn/stbu/bin/storeBackupUpdateBackup.pl" .
-			   " ... skipping"]);
-	    next;
-	}
-
-	push @out, "  $d";
-	push @dirsToCheck, $d;
-    }
-    $prLog->print('-kind' => 'I',
-		  '-str' => ["backup directories to check", @out]);
-}
 
 my $parFork = parallelFork->new('-maxParallel' => $parJobs,
 				'-prLog' => $prLog);
 my $tinySched = tinyWaitScheduler->new('-prLog' => $prLog);
 
 my $rcsf = undef;
-my ($meta, $postfix, $uncompr, @uncomprPar);
+my ($meta, $postfix, $uncompr, @uncomprPar,
+    $specialTypeArchiver, $archiveTypes);
 my $dirToCheck = undef;
 my $jobToDo = 1;
 my $parForkToDo = 0;
@@ -431,8 +430,14 @@ my $checkedFilesSizeAll = 0;
 my $linkedFilesAll = 0;
 my $linkedFilesSizeAll = 0;
 
+
+$main::wft =
+    writeBugsToFiles->new('-filePrefix' => $wrongFileTables,
+			  '-backupDir' => &::absolutePath($backupDir),
+			  '-prLog' => $prLog);
+
 # %usedInodes
-## if inode was checked, the inode is used as key, value set to 1
+## if inode was checked, the inode is used as key, value set to md5sum
 ## if md5sum check of file has different value that stored in .md5CheckSums,
 ## inode is marked as 'corrupt' and deleted from that table (so all affected
 ## files are reported)
@@ -503,7 +508,8 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 		while ($l = <IN>)
 		{
 		    chop $l;
-		    $usedBlockInodes{$l} = 1;
+		    my ($inode, $md5) = split(/\s/, $l, 2);
+		    $usedBlockInodes{$inode} = $md5;
 		}
 		last;
 	    }
@@ -582,6 +588,10 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 	    $logInBackupDirFileName .= '.bz2'
 		if $compressLogInBackupDir eq 'yes';
 	     ($uncompr, @uncomprPar) = @{$rcsf->getInfoWithPar('uncompress')};
+	    $archiveTypes = $rcsf->getInfoWithPar('archiveTypes');
+	    $archiveTypes = '' unless $archiveTypes;
+	    $specialTypeArchiver =
+		$rcsf->getInfoWithPar('specialTypeArchiver');
 
 	    $filesFromMD5CheckSumFile{'.md5BlockCheckSums.bz2'} = 1;
 	    $filesFromMD5CheckSumFile{'.md5CheckSums.bz2'} = 1;
@@ -629,7 +639,7 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 	    my $filename = "$dirToCheck/$f";
 	    if (length($md5sum) == 32)
 	    {
-#print "-1-$compr-\n";
+#print "-1-$compr-$md5sum-\n";
 		if ($compr ne 'b')
 		{
 		    $filesFromMD5CheckSumFile{$f} = 2; # not a blocked file
@@ -643,10 +653,30 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 #print "-3-\n";
 			    ++$linkedFiles;
 			    $linkedFilesSize += $sizeBackup;
+#print "-3.1-", ($usedInodes{$inode} ? $usedInodes{$inode} : 'undef') , "\n";
+			    if ($usedInodes{$inode} ne $md5sum)
+			    {
+				$prLog->print('-kind' => 'E',
+					      '-str' =>
+	              ["calculated md5 sum of <$dirToCheck/$f> is " .
+		       "different from the one in " .
+		       "<$dirToCheck/.md5BlockCheckSums> (1)"]);
+				$main::wft->print("$dirToCheck/$f",
+						  'md5Wrong');
+			    }
+			    else
+			    {
+				$usedInodes{$inode} = $md5sum;
+			    }
+#print "-3.2-", $usedInodes{$inode}, "\n";
 			    next;
 			}
-			$usedInodes{$inode} = 1;
-		    }
+			else
+			{
+#print "-3.3-", ($usedInodes{$inode} ? $usedInodes{$inode} : 'undef') , "\n";
+			    $usedInodes{$inode} = $md5sum;
+#print "-3.4-", $usedInodes{$inode}, "\n";
+			}		    }
 		}
 #print "-3.5-filesFromMD5CheckSumFile\{$f\}=", $filesFromMD5CheckSumFile{$f}, "-\n";
 #print "\tdevInode=<$devInode>\n";
@@ -654,6 +684,7 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 		{
 		    $prLog->print('-kind' => 'E',
 				  '-str' => ["file <$filename> is missing"]);
+		    $main::wft->print($filename, 'fileMissing');
 #print "-4-\n";
 		    next;
 		}
@@ -698,45 +729,54 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 		$prLog->print('-kind' => 'E',
 			      '-str' => ["named pipe <$filename is missing>"])
 		    unless -e $filename;
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["<$filename> is not a named pipe!"])
-		    unless -p $filename;
-	    }
-	    elsif ($md5sum eq 'pipe')
-	    {
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["named pipe <$filename is missing>"])
-		    unless -e $filename;
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["<$filename> is not a named pipe!"])
-		    unless -p $filename;
+		unless ($specialTypeArchiver and
+		    $archiveTypes =~ /p/)
+		{
+		    $prLog->print('-kind' => 'E',
+				  '-str' => ["<$filename> is not a named pipe!"])
+			unless -p $filename;
+		}
 	    }
 	    elsif ($md5sum eq 'socket')
 	    {
 		$prLog->print('-kind' => 'E',
 			      '-str' => ["socket <$filename> is missing"])
 		    unless -e $filename;
-		$prLog->print('-kind' => 'E',
+		unless ($specialTypeArchiver and
+		    $archiveTypes =~ /S/)
+		{
+		    $prLog->print('-kind' => 'E',
 			      '-str' => ["<$filename> is not a socket!"])
 		    unless -S $filename;
+		}
 	    }
 	    elsif ($md5sum eq 'blockdev')
 	    {
 		$prLog->print('-kind' => 'E',
 			      '-str' => ["block device <$filename> is missing"])
 		    unless -e $filename;
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["<$filename> is not a block device!"])
-		    unless -b $filename;
+		unless ($specialTypeArchiver and
+		    $archiveTypes =~ /p/)
+		{
+		    $prLog->print('-kind' => 'E',
+				  '-str' =>
+				  ["<$filename> is not a block device!"])
+			unless -b $filename;
+		}
 	    }
 	    elsif ($md5sum eq 'chardev')
 	    {
 		$prLog->print('-kind' => 'E',
 			      '-str' => ["char device <$filename> is missing"])
 		    unless -e $filename;
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["<$filename> is not a char device!"])
-		    unless -c $filename;
+		unless ($specialTypeArchiver and
+		    $archiveTypes =~ /p/)
+		{
+		    $prLog->print('-kind' => 'E',
+				  '-str' =>
+				  ["<$filename> is not a char device!"])
+			unless -c $filename;
+		}
 	    }
 
 #print "-10-$f\n";
@@ -802,6 +842,7 @@ else
 }
 
 
+
 if ($prLog->encountered('-kind' => "E"))
 {
     exit 1;
@@ -851,6 +892,7 @@ sub checkMD5
 	    {
 		$prLog->print('-kind' => 'E',
 			      '-str' => ["file <$f> is missing"]);
+		$main::wft->print($f, 'fileMissing');
 		++$nrErrors;
 		print OUT "errors $nrErrors\n";
 		return 1;
@@ -951,16 +993,32 @@ sub checkMD5
 				   ":", "\t<$l>"],
 				  '-exit' => 1)
 		}
+
 		if (-e "$dirToCheck/$l_f")
 		{
 		    my $inode = (stat("$dirToCheck/$l_f"))[1];
-		    $inode2md5{$inode} = $l_md5;
-#print "$l_f: $inode -> $l_md5\n";
+		    if (exists $usedBlockInodes{$inode} and
+			$usedBlockInodes{$inode} ne $l_md5)
+		    {
+			$prLog->print('-kind' => 'E',
+				      '-str' =>
+				      ["calculated md5 sum of <$dirToCheck/$l_f> " .
+				      "is different from the one in the checksum " .
+				       "file <$f/.md5BlockCheckSums.bz2> (3)"]);
+			$inode2md5{$inode} = $$usedBlockInodes{$inode};
+			++$nrErrors;
+		    }
+		    else
+		    {
+			$inode2md5{$inode} = $l_md5;
+		    }
 		}
 		else
 		{
 		    $prLog->print('-kind' => 'E',
-				  '-str' => ["file <$l_f> is missing"]);
+				  '-str' => ["file <$dirToCheck/$l_f> is missing"]);
+		    $main::wft->print("$dirToCheck/$l_f", 'fileMissing');
+		    ++$nrErrors;
 		}
 	    }
 	    $fileIn->close();
@@ -974,7 +1032,8 @@ sub checkMD5
 		my $calcMD5 = 0;
 		if (exists $$usedBlockInodes{$inode})
 		{
-#print "inode already calculated: $inode\n";
+#print "+++++++++inode already calculated: $inode = ",
+# $usedBlockInodes{$inode}, "\n";
 		    ++$linkedFiles;
 		    $linkedFilesSize += $size;
 		}
@@ -984,7 +1043,6 @@ sub checkMD5
 		    $checkedFilesSize += $size;
 		    $calcMD5 = 1;
 		}
-		$$usedBlockInodes{$inode} = $inode;
 
 		local *FROM;
 		my $fileIn = undef;
@@ -1027,27 +1085,33 @@ sub checkMD5
 		}
 
 		my $digest = $md5Block->hexdigest() if $calcMD5;
+		if ($calcMD5)
+		{
+		    $$usedBlockInodes{$inode} = $digest;
+		}
 #print "$f/$entry:\n";
 #print "\t$digest = digest\n";
-#print "\t", $inode2md5{$inode}, " = inode ($inode)\ if $calcMD5;n";
+#print "\t", $inode2md5{$inode}, " = inode ($inode)\ if $calcMD5\n";
 		if (not exists $inode2md5{$inode})
 		{
 		    ++$nrErrors;
-		    print OUT "errors $nrErrors\n";
+#		    print OUT "errors $nrErrors\n";
 		    $prLog->print('-kind' => 'E',
 				  '-str' =>
 				  ["<$f/$entry> is missing in " .
 				   "<$f/.md5BlockCheckSums.bz2>"]);
+		    $main::wft->print("$f/$entry", 'md5Missing');
 		}
 		elsif ($calcMD5 and $digest ne $inode2md5{$inode})
 		{
 		    ++$nrErrors;
-		    print OUT "errors $nrErrors\n";
 		    $prLog->print('-kind' => 'E',
 				  '-str' =>
 				  ["calculated md5 sum of <$f/$entry> is " .
 				   "different from the one in " .
-				   "<$f/.md5BlockCheckSums.bz2>"]);
+				   "<$f/.md5BlockCheckSums.bz2> (2)"]);
+		    $main::wft->print("$f/$entry", 'md5Wrong');
+
 		}
 	    }
 
@@ -1075,10 +1139,11 @@ sub checkMD5
     my $i;
     foreach $i (keys %inode2md5)
     {
-	print OUT "$i\n";
+	print OUT "$i ", $inode2md5{$i}, "\n";
     }
 
     close(OUT);
+#    system("cat $tmpName");
     return 0;
 }
 
@@ -1111,9 +1176,13 @@ sub _checkAllFiles
     if ($rel)
     {
 #print "\t-4- check1-> <$rel>\n";
-	$prLog->print('-kind' => 'E',
-		      '-str' => ["<$rel> is not listed in .md5CheckSum (1)"])
-	    unless exists $$relFiles{$rel};
+	unless (exists $$relFiles{$rel})
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' =>
+			  ["<$dir/$rel> is not listed in .md5CheckSum (1)"]);
+	    $main::wft->print("$dir/$rel", 'md5Missing');
+	}
     }
 
     local *DIR;
@@ -1135,9 +1204,13 @@ sub _checkAllFiles
 
 	$rel = substr($de, $length);
 #print "\t-10- set2 rel <$rel>\n";
-	$prLog->print('-kind' => 'E',
-		      '-str' => ["<$rel> is not listed in .md5CheckSum (2)"])
-	    unless exists $$relFiles{$rel};
+	unless (exists $$relFiles{$rel})
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' =>
+			  ["<$dir/$rel> is not listed in .md5CheckSum (2)"]);
+	    $main::wft->print("$dir/$rel", 'md5Missing');
+	}
     }
     closedir(DIR) or
 	$prLog->print('-kind' => 'E',
