@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2008-2012)
+#   Copyright (C) Dr. Heinz-Josef Claes (2008-2013)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -120,7 +120,7 @@ storeBackupUpdateBackup.pl - updates / finalizes backups created by storeBackup.
 	      [--dontCopyBackup] [-A archiveDurationDeltaCache]
 	      [--dontDelInDeltaCache]
 	      [--logFile
-	       [--suppressTime] [-m maxFilelen]
+	       [--plusLogStdout] [--suppressTime] [-m maxFilelen]
 	       [[-n noOfOldFiles] | [--saveLogs]]
 	       [--compressWith compressprog]]
 
@@ -225,6 +225,11 @@ storeBackupUpdateBackup.pl - updates / finalizes backups created by storeBackup.
 
     logFile, Default: stdout
 
+=item B<--plusLogStdout>
+
+    if you specify a log file with --logFile you can
+    additionally print the output to STDOUT with this flag
+
 =item B<--suppressTime>
 
     suppress output of time in logfile
@@ -261,12 +266,12 @@ storeBackupUpdateBackup.pl - updates / finalizes backups created by storeBackup.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008-2012 by Heinz-Josef Claes.
+Copyright (c) 2008-2013 by Heinz-Josef Claes.
 Published under the GNU General Public License v3 or any later version
 
 =cut
 
-my $Help = join('', grep(!/^\s*$/, `pod2text $0`));
+my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
 $Help = "cannot find pod2text, see documentation for details\n"
     unless $Help;
 
@@ -331,6 +336,9 @@ my $CheckPar =
 					    '-cl_alias' => '--logFile',
 					    '-param' => 'yes',
 					    '-only_if' => 'not [interactive]'),
+				Option->new('-name' => 'plusLogStdout',
+					    '-cl_option' => '--plusLogStdout',
+					    '-only_if' => '[logFile]'),
 				Option->new('-name' => 'suppressTime',
 					    '-cl_option' => '--suppressTime'),
 				Option->new('-name' => 'maxFilelen',
@@ -361,7 +369,15 @@ my $CheckPar =
 				Option->new('-name' => 'genDeltaCacheConf',
 					    '-cl_option' => '--genDeltaCacheConf',
 		    '-only_if' => 'not [backupDir] and not [genBackupBaseTreeConf]',
-					    '-param' => 'yes')
+					    '-param' => 'yes'),
+# hidden options
+# used by storeBackupMount.pl
+				Option->new('-name' => 'writeToNamedPipe',
+					    '-cl_option' => '--writeToNamedPipe',
+					    '-param' => 'yes',
+					    '-hidden' => 'yes'),
+				Option->new('-name' => 'skipSync',
+					    '-cl_option' => '--skipSync')
 				]
 		    );
 
@@ -384,6 +400,7 @@ $archiveDurationDeltaCache = $CheckPar->getOptWithPar('archiveDurationDeltaCache
 my $dontDelInDeltaCache = $CheckPar->getOptWithoutPar('dontDelInDeltaCache');
 my $progressReport = $CheckPar->getOptWithPar('progressReport');
 my $logFile = $CheckPar->getOptWithPar('logFile');
+my $plusLogStdout = $CheckPar->getOptWithoutPar('plusLogStdout');
 my $withTime = not $CheckPar->getOptWithoutPar('suppressTime');
 $withTime = $withTime ? 'yes' : 'no';
 my $maxFilelen = $CheckPar->getOptWithPar('maxFilelen');
@@ -393,6 +410,9 @@ my $compressWith = $CheckPar->getOptWithPar('compressWith');
 my $backupDir = $CheckPar->getOptWithPar('backupDir');
 my $genBackupBaseTreeConf = $CheckPar->getOptWithPar('genBackupBaseTreeConf');
 my $genDeltaCacheConf = $CheckPar->getOptWithPar('genDeltaCacheConf');
+# hidden options
+my $writeToNamedPipe = $CheckPar->getOptWithPar('writeToNamedPipe');
+my $skipSync = $CheckPar->getOptWithoutPar('skipSync');
 
 
 unless ($noCompress)
@@ -555,13 +575,15 @@ my ($prLogKind) = ['A:BEGIN',
 		   'P:PROGRESS',
 		   'S:STATISTIC',
 		   'D:DEBUG'];
-my $prLog = printLog->new('-kind' => $prLogKind,
-			  @par,
-			  '-withTime' => $withTime,
-			  '-maxFilelen' => $maxFilelen,
-			  '-noOfOldFiles' => $noOfOldFiles,
-			  '-saveLogs' => $saveLogs,
-			  '-compressWith' => $compressWith);
+my $prLog1 = printLog->new('-kind' => $prLogKind,
+			   @par,
+			   '-withTime' => $withTime,
+			   '-maxFilelen' => $maxFilelen,
+			   '-noOfOldFiles' => $noOfOldFiles,
+			   '-saveLogs' => $saveLogs,
+			   '-compressWith' => $compressWith);
+
+my $prLog = printLogMultiple->new('-prLogs' => [$prLog1]);
 $prLog->fork($req);
 
 if ($print)
@@ -570,12 +592,42 @@ if ($print)
     exit 0;
 }
 
+if ($plusLogStdout)
+{
+    my $p = printLog->new('-kind' => $prLogKind,
+			  '-filedescriptor', *STDOUT);
+    $prLog->add('-prLogs' => [$p]);
+}
+if ($writeToNamedPipe)
+{
+    my $pl = $prLog;
+    my $np = printLog->new('-kind' => $prLogKind,
+			   '-file' => $writeToNamedPipe,
+			   '-maxFilelen' => 0);
+    $prLog = printLogMultiple->new('-prLogs' => [$pl, $np]);
+}
+
 if ($interactive)
 {
     $verbose = 1;
     $debug = 1;
 }
 
+$main::IOCompressDirect = 0;
+eval "use IO::Compress::Bzip2 qw(bzip2)";
+if ($@)
+{
+    $prLog->print('-kind' => 'I',
+		  '-str' =>
+		  ["please install IO::Compress::Bzip2 from " .
+		   "CPAN for better performance"]);
+}
+else
+{
+    $main::IOCompressDirect = 1;
+}
+
+
 $prLog->print('-kind' => 'A',
 	      '-str' =>
 	      ["checking references and backup copying in <$backupDir>"]);
@@ -744,7 +796,7 @@ if (-e "$backupDir/$baseTreeConf" and not $dontCopyBackup)
 			      ["copying <$deltaCache/$s/$d> to " .
 			       "<$backupDir/$s>"]);
 		&::copyDir("$deltaCache/$s/$d" => "$backupDir/$s",
-			   "/tmp/stbuUpdateBackup-", $prLog);
+			   "/tmp/stbuUpdateBackup-", $prLog, 0);
 		open(FILE, '>>', "$deltaCache/$s/$d.copied") or
 		    $prLog->print('-kind' => 'E',
 				  '-str' =>
@@ -807,7 +859,7 @@ if (-e "$backupDir/$baseTreeConf" and not $dontCopyBackup)
 				  '-exit' => 1);
 		close(FILE);
 		&::copyDir("$backupDir/$s/$entry" => "$deltaCache/$s",
-			   "/tmp/stbuUpdateBackup-", $prLog);
+			   "/tmp/stbuUpdateBackup-", $prLog, 0);
 		unlink "$deltaCache/$s/$entry.notFinished";
 	    }
 	}
@@ -1311,9 +1363,12 @@ $prLog->print('-kind' => 'S',
 
 unlink $lockFile;
 
-$prLog->print('-kind' => 'I',
-	      '-str' => ["syncing ..."]);
-system "/bin/sync";
+unless ($skipSync)
+{
+    $prLog->print('-kind' => 'I',
+		  '-str' => ["syncing ..."]);
+    system "/bin/sync";
+}
 
 $prLog->print('-kind' => 'Z',
 	      '-str' =>
@@ -1322,6 +1377,16 @@ $prLog->print('-kind' => 'Z',
 exit 0;
 
 
+##################################################
+# package printLogMultiple needs this function
+sub cleanup
+{
+    my $signame = shift;
+    my $exit = (shift() || 1);
+
+    exit $exit;
+}
+
 
 ############################################################
 sub updateBackupDir
@@ -1452,34 +1517,62 @@ sub updateBackupDir
 	    $file = "$dir/$file";        # file to compress
 	    my $st = (stat($file))[7];
 	    $oldSize += $st if defined $st;
-	    my ($old, $new) =
-		$parForkProc->add_block('-exec' => $compr,
-					'-param' => \@comprPar,
-					'-outRandom' => '/tmp/bzip2-',
-					'-stdin' => $file,
-					'-stdout' => "$file$comprPostfix",
-					'-delStdout' => 'no',
-					'-info' => $file);
-	    if ($old)
-	    {
-		$noCompressedFiles++;
-		$prLog->print('-kind' => 'S',
-			      '-str' => ["compressed $noCompressedFiles files"])
-		    if ($progressReport and
-			$noCompressedFiles % $progressReport == 0);
+	    $noCompressedFiles++;
 
-		my $f = $old->get('-what' => 'info');
-		&::waitForFile("$f$comprPostfix");
-		$newSize += (stat("$f$comprPostfix"))[7];
-		my $out = $old->getSTDERR();
+	    if ($main::IOCompressDirect and
+		$compr eq 'bzip2')
+	    {
+		local *FILEIN;
+		sysopen(FILEIN, $file, O_RDONLY) or
+		    $prLog->print('-kind' => 'E',
+				  '-str' =>
+				  ["cannot open <$file> for compression"]);
+		my $bz = new IO::Compress::Bzip2("$file$comprPostfix",
+						 BlockSize100K => 9);
+		my $buffer;
+		while (sysread(FILEIN, $buffer, 1025))
+		{
+		    $bz->syswrite($buffer);
+		}
+		$bz->flush();
+		$bz->eof();
+		close(FILEIN);
+
+		$newSize += (stat("$file$comprPostfix"))[7];
 		$prLog->print('-kind' => 'E',
-			      '-str' => ["STDERR of <$compr @comprPar " .
-					 "<$f >$f$comprPostfix>:", @$out])
-		    if (@$out > 0);
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["cannot delete <$f>"])
-		    if (unlink $f) != 1;
+			      '-str' => ["cannot delete <$file>"])
+		    if (unlink $file) != 1;
 	    }
+	    else
+	    {
+		my ($old, $new) =
+		    $parForkProc->add_block('-exec' => $compr,
+					    '-param' => \@comprPar,
+					    '-outRandom' => '/tmp/bzip2-',
+					    '-stdin' => $file,
+					    '-stdout' => "$file$comprPostfix",
+					    '-delStdout' => 'no',
+					    '-info' => $file);
+		if ($old)
+		{
+		    my $f = $old->get('-what' => 'info');
+		    &::waitForFile("$f$comprPostfix");
+		    my $out = $old->getSTDERR();
+		    $prLog->print('-kind' => 'E',
+				  '-str' => ["STDERR of <$compr @comprPar " .
+					     "<$f >$f$comprPostfix>:", @$out])
+			if (@$out > 0);
+		    $newSize += (stat("$f$comprPostfix"))[7];
+		    $prLog->print('-kind' => 'E',
+				  '-str' => ["cannot delete <$f>"])
+			if (unlink $f) != 1;
+		}
+	    }
+	    $prLog->print('-kind' => 'S',
+			  '-str' =>
+			  ["compressed $noCompressedFiles files"])
+		if ($progressReport and
+		    $noCompressedFiles % $progressReport == 0);
 	}
 	else
 	{
@@ -1517,13 +1610,15 @@ sub updateBackupDir
 	    if (unlink $f) != 1;
     }
 
+    my $percent = $oldSize ? $newSize / $oldSize * 100 : 0;
     $prLog->print('-kind' => 'S',
 		  '-str' => ["created $noMkdir directories",
 			     "created $noSymLink symbolic links",
 			     "compressed $noCompressedFiles files",
 			     "used " . (&::humanReadable($newSize))[0] .
 			     " instead of " . (&::humanReadable($oldSize))[0] .
-			     " ($newSize <- $oldSize)"]);
+			     " ($newSize <- $oldSize ; " .
+			     (sprintf "%.1f", $percent) . "%)"]);
 
     #
     # set hard links
@@ -1667,7 +1762,8 @@ sub updateBackupDir
 	    $newLink =~ s/\0/\n/og;
 
 	    $noHardLinks +=
-		&::hardLinkDir($existingFile, $newLink, '\A\d.*', $prLog);
+		&::hardLinkDir($existingFile, $newLink, '\A\d.*',
+			       undef, undef, undef, $prLog);
 
 	    if ($withBlockedFiles)
 	    {
@@ -1746,13 +1842,11 @@ sub updateBackupDir
     #
     my $preservePerms =
 	$rcsf->getInfoWithPar('preservePerms') eq 'no' ? 0 : 1;
-#	(($$meta{'preservePerms'})->[0] eq 'no') ? 0 : 1;
     $pr = $progressReport * 2000;
     if ($preservePerms)
     {
 	$prLog->print('-kind' => 'I',
 		      '-str' => ["phase 3: setting file permissions"]);
-#	my $comprPostfix = ($$meta{'postfix'})->[0];
 	my $comprPostfix = $rcsf->getInfoWithPar('postfix');
 
 	my $noFiles = 0;
@@ -1785,6 +1879,29 @@ sub updateBackupDir
 	    chown $uid, $gid, $file;
 	    if ($compr eq 'b')         # block file
 	    {
+		local *BFDIR;
+		opendir(BFDIR, $file) or
+		    $prLog->print('-kind' => 'E',
+				  '-str' =>
+				  ["cannot opendir <$file> to set permissions"],
+				  '-add' => [__FILE__, __LINE__]);
+		my $bfentry;
+		while ($bfentry = readdir BFDIR)
+		{
+		    if ($bfentry =~ /\A\.md5BlockCheckSums/)
+		    {
+			chmod 0644, "$file/$bfentry";
+		    }
+		    else
+		    {
+			utime $atime, $mtime, "$file/$bfentry";
+			chown $uid, $gid, "$file/$bfentry";
+			chmod $mode, "$file/$bfentry";
+		    }
+		    $noFiles++;
+		}
+		closedir(BFDIR);
+
 		$mode &= 0777;    # strip special permissions
 		$mode |= 0111;    # add directory permissions
 	    }
