@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2009)
+#   Copyright (C) Heinz-Josef Claes (2001-2013)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -20,9 +20,9 @@
 
 
 
-push @VERSION, '$Id: fileDir.pl 357 2009-05-02 15:05:33Z hjc $ ';
-
+use Digest::MD5 qw(md5_hex);
 use Fcntl qw(O_RDWR O_CREAT);
+use Fcntl ':mode';
 use POSIX;
 
 require 'prLog.pl';
@@ -30,6 +30,153 @@ require 'forkProc.pl';
 
 use strict;
 
+
+# copies a file
+# returns:
+#          1: ok
+#          0: error
+############################################################
+sub copyFile
+{
+    my $source = shift;
+    my $target = shift;
+
+    local *SOURCE;
+    local *TARGET;
+    unless (sysopen(SOURCE, $source, O_RDONLY))
+    {
+	return 0;
+    }
+    unless (sysopen(TARGET, $target, O_CREAT | O_WRONLY))
+    {
+	return 0;
+    }
+    my $buffer;
+    while (sysread(SOURCE, $buffer, 10*1024**2))
+    {
+	syswrite(TARGET, $buffer);
+    }
+
+    close(TARGET);
+    close(SOURCE);
+
+    return 1;
+}
+
+############################################################
+sub copySymLink
+{
+    my $source = shift;    # symlink source
+    my $target = shift;    # symlink target
+
+    my $l = readlink $source;
+    unless ($l)
+    {
+	return 0;
+    }
+    unless (symlink $l, $target)
+    {
+	return 0;
+    }
+    else
+    {
+	return 1;
+    }
+}
+
+
+############################################################
+# use cp_a to copy a directory
+# $ignoreError = 1 -> ignore error
+# $ignoreError = 0 -> stop at error
+sub copyDir
+{
+    my ($from, $to, $tmp, $prLog, $ignoreError) = (@_);
+
+    my $cp = forkProc->new('-exec' => 'cp',
+			   '-param' =>
+			   ['-r', "$from", "$to"],
+			   '-outRandom' => $tmp,
+			   '-prLog' => $prLog);
+    $cp->wait();
+    my $out = $cp->getSTDOUT();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'W',
+		      '-str' =>
+		      ["copying of <$from> to <$to> reported:",
+		       @$out]);
+    }
+    $out = $cp->getSTDERR();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["copying (with 'cp -r') of <$from> to <$to> reported:",
+		       @$out]);
+	exit 1 unless $ignoreError;
+    }
+}
+
+
+############################################################
+# use mv to move a directory
+sub mvComm
+{
+    my ($from, $to, $tmp, $prLog) = (@_);
+
+    my $cp = forkProc->new('-exec' => 'mv',
+			   '-param' =>
+			   ["$from", "$to"],
+			   '-outRandom' => $tmp,
+			   '-prLog' => $prLog);
+    $cp->wait();
+    my $out = $cp->getSTDOUT();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'W',
+		      '-str' =>
+		      ["moving of <$from> to <$to> reported:",
+		       @$out]);
+    }
+    $out = $cp->getSTDERR();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["moving of <$from> to <$to> reported:",
+		       @$out],
+		      '-exit' => 1);
+    }
+}
+
+
+
+############################################################
+# &::readDirStbu($dir, __FILE__, __LINE__, $prLog);
+sub readDirStbu
+{
+    my ($dir, $file, $line, $pattern, $prLog) = (@_);
+
+    local *DIR;
+    opendir(DIR, "$dir") or
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["cannot opendir <$dir>, exiting"],
+		      '-add' => [$file, $line],
+		      '-exit' => 1);
+    my (@dirs, $entry);
+    while ($entry = readdir DIR)
+    {
+	next unless $entry =~ /$pattern/;
+	push @dirs, $entry;
+    }
+    closedir(DIR);
+
+    return (sort @dirs);
+}
+
+
 # checks if a file is a symlink and deletes it if wanted
 # return values (if not exiting):
 #               0: no symlink
@@ -41,7 +188,7 @@ sub checkDelSymLink
     my $prLog = shift;
     my $delExit = shift;    # set bits:
                             #  bit 0:  0 = do not delete
-                            #          1 = delete synlink
+                            #          1 = delete symlink
                             #              if not possible, exit
                             #  bit 1:  0 = do not exit (if exists)
                             #          1 = exit
@@ -76,16 +223,16 @@ sub splitFileDir
     my $name = shift;
 
     return (undef, undef) unless $name;
-    return ('.', $name) unless ($name =~/\//);    # nur einfacher Dateiname
+    return ('.', $name) unless ($name =~/\//);    # simple file name only
 
     my ($dir, $file) = $name =~ /^(.*)\/(.*)$/s;
-    $dir = '/' if ($dir eq '');                   # gilt, falls z.B. /filename
+    $dir = '/' if ($dir eq '');                   # if eg. /filename
     return ($dir, $file);
 }
 
 
 ############################################################
-# Parameter kann directory oder Datei sein
+# Parameter may be directory or file
 %main::absolutePathCache = ();
 @main::absolutePathCache = ();
 sub absolutePath
@@ -94,16 +241,16 @@ sub absolutePath
 
     return undef unless $dir;
 
-    # Erst im Hash nachsehen, ob das schon ermittelt wurde
+    # First look up in the hash table if that's already calculated
     return $main::absolutePathCache{$dir}
         if exists $main::absolutePathCache{$dir};
     my $dirSave = $dir;
 
-    # Falls Datei selbst ein symlink ist, solange folgen, bis aufgelst
+    # if file itself is a hard link, follow the hard link(s) to something real
     my $file = '';
 
 
-    $dir = $ENV{'PWD'} . "/$dir" unless ($dir =~ /^\//); # relativer Pfad
+    $dir = $ENV{'PWD'} . "/$dir" unless ($dir =~ /^\//); # relative path
 
     if (-e $dir)
     {
@@ -127,11 +274,11 @@ sub absolutePath
     }
 
     my $ret = (-e "$dir$file") ? "$dir$file" : undef;
-    $ret =~ s/\/\//\//g;       # // -> /
-    $ret =~ s/\/\.\//\//g;     # /./ -> /
-    $ret =~ s/\A(.+)\/\Z/$1/;  # remove trailing /
-    $ret =~ s/\A\.\///;        # ./xxx -> xxx
-    $ret =~ s/\/\.\Z//;        # xxx/. -> xxx
+    $ret =~ s/\/+/\//g;                # // -> /
+    while ($ret =~ s/\/\.\//\//g) {}   # /./ -> /
+    $ret =~ s/\A(.+?)\/+\Z/$1/;        # remove trailing /
+    $ret =~ s/\A\.\///;                # ./xxx -> xxx
+    $ret =~ s/\/\.\Z//;                # xxx/. -> xxx
 
     # eliminate /../
     my (@ret) = split(/\//, $ret);
@@ -204,7 +351,7 @@ sub relPath
 ############################################################
 sub uniqFileName
 {
-    my $prefix = shift;                 # z.B. '/tmp/test-'
+    my $prefix = shift;                 # eg. '/tmp/test-'
 
     my $suffix;
     do
@@ -218,7 +365,7 @@ sub uniqFileName
 
 
 ############################################################
-# testet, ob subDir ein Unterverzeichnis von dir ist
+# tests if subDir is a sub directory of dir
 sub isSubDir
 {
     my $dir = shift;
@@ -265,17 +412,21 @@ sub makeFilePath
     my $prLog = shift;
 
     $path =~ m#\A(.*)/.*\Z#s;
-    &makeDirPath($1);
+    &makeDirPath($1, $prLog);
 }
 
 
 ############################################################
 # like `mkdir -p`, all permissions set to 0700
+# success: returns 1
+# no success: returns 0
 sub makeDirPath
 {
     my $path = shift;
     my $prLog = shift;
 
+    return unless $path;
+
     my @p;
     return if -e $path;
 
@@ -283,12 +434,14 @@ sub makeDirPath
 
     &makeDirPath($p1, $prLog);
 
-    $prLog->print('-kind' => 'E',
-		  '-str' => ["cannot create directory <$path>"],
-		  '-exit' => 1)
-	unless mkdir $path, 0700;
-
-    return;
+    unless (mkdir $path, 0700)
+    {
+	return if -e $path;
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot create directory <$path>"]);
+	return 0;
+    }
+    return 1;
 }
 
 
@@ -338,7 +491,7 @@ sub checkLockFile
     local *FILE;
     if (-f $lockFile)
     {
-	open(FILE, "< $lockFile") or
+	open(FILE, '<', $lockFile) or
 	    $prLog->print('-kind' => 'E',
 			  '-str' => ["cannot read lock file <$lockFile>"],
 			  '-exit' => 1);
@@ -369,7 +522,7 @@ sub checkLockFile
 		  '-str' => ["creating lock file <$lockFile>"]);
 
     &::checkDelSymLink($lockFile, $prLog, 0x01);
-    open(FILE, "> $lockFile") or
+    open(FILE, '>', $lockFile) or
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["cannot create lock file <$lockFile>"],
 		      '-exit' => 1);
@@ -400,6 +553,144 @@ sub calcMD5
 }
 
 
+############################################################
+sub readSymLinkCalcMd5
+{
+    my $filename = shift;
+    my $prLog = shift;
+
+    my $l = readlink $filename;
+    unless ($l)
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot read symlink value <$filename>"]);
+	return undef;
+    }
+    my $md5 = Digest::MD5->new();
+    $md5->add($l);
+    return $md5->hexdigest();
+}
+
+############################################################
+sub createArchiveFromFile
+{
+    my $filename = shift;   # relative path
+    my $sourceDir = shift;
+    my $backupDir = shift;
+    my $archiver = shift;
+    my $prLog = shift;
+
+    my $prog = $main::fileTypeArchiver{$archiver}{'prog'};
+    my $opts = $main::fileTypeArchiver{$archiver}{'createOpts'}; # \@list
+    my ($pathToSourceFile, $f) = &::splitFileDir("$sourceDir/$filename");
+
+    if ($archiver eq 'tar')
+    {
+	push @$opts, '-', $f;
+	my $tar = forkProc->new('-exec' => $prog,
+				'-param' => $opts,
+				'-workingDir' => $pathToSourceFile,
+				'-stdout' => "$backupDir/$filename",
+				'-stderr' => '/tmp/stbuPipeArchive-',
+				'-delStdout' => 'no',
+				'-prLog' => $prLog);
+	$tar->wait();
+	my $out = $tar->getSTDOUT();
+	$prLog->print('-kind' => 'W',
+		      '-str' => ["STDOUT of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+	$out = $tar->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    elsif ($archiver eq 'cpio')
+    {
+	my $cpio = pipeToFork->new('-exec' => $prog,
+				   '-param' => $opts,
+				   '-workingDir' => $pathToSourceFile,
+				   '-stdout' => "$backupDir/$filename",
+				   '-stderr' => '/tmp/stbuPipeArchive-',
+				   '-delStdout' => 'no',
+				   '-prLog' => $prLog);
+	$cpio->print("$f\n");
+	$cpio->wait();
+	my $out = $cpio->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    else
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["archiver <$archiver> unknown, exiting"],
+		      '-exit' => 1);
+    }
+}
+
+
+############################################################
+sub extractFileFromArchive
+{
+    my $filename = shift;   # relative path
+    my $restoreDir = shift;
+    my $backupDir = shift;
+    my $archiver = shift;
+    my $prLog = shift;
+
+#print "+++extractFileFromArchive:+$filename+$restoreDir+$backupDir+$archiver+\n";
+    my $prog = $main::fileTypeArchiver{$archiver}{'prog'};
+    my $opts = $main::fileTypeArchiver{$archiver}{'extractOpts'}; # \@list
+    my ($pathToRestoreDir, $f) = &::splitFileDir("$restoreDir/$filename");
+
+    if ($archiver eq 'tar')
+    {
+	push @$opts, "$backupDir/$filename";
+#print "+++$prog @$opts+++\n";
+	my $tar = forkProc->new('-exec' => $prog,
+				'-param' => $opts,
+				'-workingDir' => $pathToRestoreDir,
+				'-outRandom' => '/tmp/stbuPipeArchive-',
+				'-prLog' => $prLog);
+	$tar->wait();
+	my $out = $tar->getSTDOUT();
+	$prLog->print('-kind' => 'W',
+		      '-str' => ["STDOUT of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+	$out = $tar->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    elsif ($archiver eq 'cpio')
+    {
+#print "+++$prog @$opts+++\n";
+	my $cpio = forkProc->new('-exec' => $prog,
+				 '-param' => $opts,
+				 '-workingDir' => $pathToRestoreDir,
+				 '-stdin' => "$backupDir/$filename",
+				 '-outRandom' => '/tmp/stbuPipeArchive-',
+				 '-prLog' => $prLog);
+	$cpio->wait();
+	my $out = $cpio->getSTDOUT();
+	$prLog->print('-kind' => 'W',
+		      '-str' => ["STDOUT of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+	$out = $cpio->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    else
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["archiver <$archiver> unknown, exiting"],
+		      '-exit' => 1);
+    }
+#print "+++end of extractFileFromArchive:+$filename+\n";
+}
+
+
 ############################################################
 # Objekt kann zum (wiederholten) Abfragen von Informationen
 # ber eine Datei verwendet werden.
@@ -682,15 +973,7 @@ sub readDir
     local *DIR;
     unless (opendir(DIR, $dir))
     {
-	if ($ignoreReadError eq 'no')
-	{
-	    $prLog->print('-kind' => $prLogErr,
-			  '-str' =>
-			  ["cannot opendir <$dir>, exiting",
-			   "check your source and clean your backup!"],
-			  '-exit' => $exit);
-	}
-	elsif ($ignoreReadError eq 'onlyPrintMessage')
+	if ($ignoreReadError)
 	{
 	    $prLog->print('-kind' => $prLogErr,
 			  '-str' => ["cannot opendir <$dir>"]);
@@ -698,7 +981,11 @@ sub readDir
 	}
 	else
 	{
-	    return;
+	    $prLog->print('-kind' => $prLogErr,
+			  '-str' =>
+			  ["cannot opendir <$dir>, exiting",
+			   "check your source and clean your backup!"],
+			  '-exit' => $exit);
 	}
     }
 
@@ -708,6 +995,9 @@ sub readDir
     {
 	next if ($entry eq '.' or $entry eq '..');
 	$entry = $dir . '/' . $entry;
+
+#	my $mode = (stat($entry))[2];
+#	if (::S_ISLNK($mode))
 	if (-l $entry)
 	{
 	    if ($self->{'followLinks'} > $depth and
@@ -735,12 +1025,23 @@ sub readDir
 	    next;
 	}
 	unless (-r $entry)
+#	unless ($mode & ::S_IREAD and $> != 0)  # $> -> effective user id
 	{
-	    $prLog->print('-kind' => $prLogWarn,
-			  '-str' => ["no permissions to read <$entry>"]);
+	    if ($ignoreReadError)
+	    {
+		$prLog->print('-kind' => $prLogWarn,
+			      '-str' => ["no permissions to read <$entry>"]);
+	    }
+	    else
+	    {
+		$prLog->print('-kind' => $prLogErr,
+			      '-str' => ["no permissions to read <$entry>"],
+			      '-exit' => 1);
+	    }
 	    next;
 	}
 	if (-d $entry)       # Dieses Directory mu beim Kopieren
+#	if (::S_ISDIR($mode))  # Dieses Directory mu beim Kopieren
 	{                      # z.B. mu angelegt werden!
 	    push @$files, $entry;
 	    push @$types, 'd';
@@ -750,30 +1051,35 @@ sub readDir
 	    next;
 	}
 	if (-f $entry)
+#	if (::S_ISREG($mode))
 	{
 	    push @$files, $entry;
 	    push @$types, 'f';
 	    next;
 	}
 	if (-p $entry)      # named pipe
+#	if (::S_ISFIFO($mode))      # named pipe
 	{
 	    push @$files, $entry;
 	    push @$types, 'p';
 	    next;
 	}
 	if (-S $entry)      # socket
+#	if (::S_ISSOCK($mode))      # socket
 	{
 	    push @$files, $entry;
 	    push @$types, 'S';
 	    next;
 	}
 	if (-b $entry)      # block special file
+#	if (::S_ISBLK($mode))      # block special file
 	{
 	    push @$files, $entry;
 	    push @$types, 'b';
 	    next;
 	}
 	if (-c $entry)      # character special file
+#	if (::S_ISCHR($mode))      # character special file
 	{
 	    push @$files, $entry;
 	    push @$types, 'c';
