@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2012)
+#   Copyright (C) Heinz-Josef Claes (2001-2013)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -64,17 +64,18 @@ sub waitForFile
 
     my @intervall = (.01, .01, .02, .02, .04, .05, .05,
 		      .05, .05, .05, .05, .05, .05, .05, .05,
-		     .05, .05, .05, .05, .05, .05, .05, .05); # 1 second
+		     .05, .05, .05, .05, .05, .05, .05, .05, 0); # 1 second
 
-    if (not -e $existingFile)
+    if ((not -e $existingFile) and (stat($existingFile))[7] == 0)
     {
 	my $i;
 	foreach $i (@intervall)
 	{
 	    select(undef, undef, undef, $i);
-	    if (-e $existingFile)
+	    if (-e $existingFile and (stat($existingFile))[7] > 0)
 	    {
-		select(undef, undef, undef, $intervall[0]);
+		select(undef, undef, undef, 2 * $intervall[0]);
+#		system "/bin/sync";
 		last;
 	    }
 	}
@@ -82,7 +83,7 @@ sub waitForFile
 	if ($i == 0)
 	{
 	    system "/bin/sync";
-	    return not -e $existingFile;
+	    return (not -e $existingFile) and (stat($existingFile))[7] == 0;
 	}
     }
 
@@ -731,7 +732,6 @@ sub new
 	    chown 0600, $stderr;
 	}
 
-	$proc =~ s/(\s)/\\$1/g;
 	unless (exec $proc, @$par)
 	{
 	    $prLog->print('-kind' => $err,
@@ -1048,8 +1048,7 @@ sub add_block
     my $sched =
 	tinyWaitScheduler->new('-firstFast'     => $self->{'firstFast'},
 			       '-noOfWaitSteps' => $self->{'noOfWaitSteps'},
-			       '-prLog'         => $self->{'prLog'},
-			       '-noOfWaitSteps' => 10);
+			       '-prLog'         => $self->{'prLog'});
 
     my $old;
     $sched->wait() until
@@ -1188,7 +1187,7 @@ sub new
     my $self = {};
 
     my (%params) = ('-maxWaitTime'   => 1,     # in seconds (-> add())
-		    '-noOfWaitSteps' => 10,
+		    '-noOfWaitSteps' => 100,
 		    '-firstFast'     => 1,     # falls 1, beim ersten Mal nicht warten
                     '-debug'         => 0,
                     '-prLog'         => undef,
